@@ -1,7 +1,6 @@
 """A module for asynchronously interacting with KEF wireless speakers."""
 
 import asyncio
-import contextlib
 import functools
 import inspect
 import logging
@@ -230,7 +229,7 @@ class _AsyncCommunicator:
         host: str,
         port: int,
         *,
-        ioloop: Optional[asyncio.events.AbstractEventLoop] = None,
+        loop: Optional[asyncio.events.AbstractEventLoop] = None,
     ):
         self.host = host
         self.port = port
@@ -238,7 +237,7 @@ class _AsyncCommunicator:
         self._writer: Optional[asyncio.StreamWriter] = None
         self._last_time_stamp = 0.0
         self._is_online = False
-        self._ioloop = ioloop or asyncio.get_event_loop()
+        self._loop = loop or asyncio.get_event_loop()
         self._disconnect_task = None
         self._lock = asyncio.Lock()
 
@@ -248,20 +247,20 @@ class _AsyncCommunicator:
 
     async def open_connection(self) -> None:
         if self.is_connected:
-            _LOGGER.debug("Connection is still alive")
+            _LOGGER.debug("%s: Connection is still alive", self.host)
             return
         retries = 0
         while retries < _MAX_CONNECTION_RETRIES:
-            _LOGGER.debug("Opening connection")
+            _LOGGER.debug("%s: Opening connection", self.host)
 
             try:
                 async with self._lock, timeout(_TIMEOUT):
                     self._reader, self._writer = await asyncio.open_connection(
                         self.host, self.port, family=socket.AF_INET
                     )
-                    _LOGGER.debug("Opening connection successful")
+                    _LOGGER.debug("%s: Opening connection successful", self.host)
             except ConnectionRefusedError:
-                _LOGGER.debug("Opening connection failed")
+                _LOGGER.debug("%s: Opening connection failed", self.host)
                 await asyncio.sleep(0.5)
             except BlockingIOError:  # Connection incomming
                 # XXX: I have never seen this.
@@ -283,50 +282,48 @@ class _AsyncCommunicator:
         async with self._lock:
             assert self._writer is not None
             assert self._reader is not None
-            _LOGGER.debug(f"Writing message: {str(message)}")
+            _LOGGER.debug("%s: Writing message: %s", self.host, str(message))
             self._writer.write(message)
             await self._writer.drain()
 
-            _LOGGER.debug("Reading message")
+            _LOGGER.debug("%s: Reading message", self.host)
             try:
                 async with timeout(_TIMEOUT):
                     data = await self._reader.read(100)
-                _LOGGER.debug(f"Got reply, {str(data)}")
+                _LOGGER.debug("%s: Got reply, %s", self.host, str(data))
                 self._last_time_stamp = time.time()
                 self._schedule_disconnect()
             except asyncio.TimeoutError:
-                _LOGGER.error("Timeout in waiting for reply")
+                _LOGGER.error("%s: Timeout in waiting for reply", self.host)
             finally:
                 return data
 
     async def _disconnect(self) -> None:
         if self.is_connected:
-            async with self._lock:
+            async with self._lock, timeout(_TIMEOUT):
                 assert self._writer is not None
-                _LOGGER.debug("Disconnecting")
+                _LOGGER.debug("%s: Disconnecting", self.host)
                 self._writer.close()
                 await self._writer.wait_closed()
                 self._reader, self._writer = (None, None)
 
+    async def _disconnect_in(self, dt):
+        await asyncio.sleep(dt)
+        await asyncio.shield(self._disconnect())  # ℹ️ shield it from being cancelled
+
     def _schedule_disconnect(self, dt=_KEEP_ALIVE):
         if self._disconnect_task is not None:
-            _LOGGER.debug("Cancelling the _disconnect_task")
+            _LOGGER.debug("%s: Cancelling the _disconnect_task", self.host)
             self._disconnect_task.cancel()
             self._disconnect_task = None
-        self._disconnect_task = self._ioloop.create_task(self._disconnect_in(dt))
-
-    async def _disconnect_in(self, dt) -> None:
-        """Disconnect socket after dt."""
-        with contextlib.suppress(Exception):
-            await asyncio.sleep(dt)
-            await self._disconnect()
+        self._disconnect_task = asyncio.create_task(self._disconnect_in(dt))
 
     @retry(**_SEND_MSG_RETRY_KWARGS)
     async def send_message(self, msg: bytes) -> int:
         await self.open_connection()
         raw_reply = await self._send_message(msg)
         reply = _parse_response(msg, raw_reply)[-2]
-        _LOGGER.debug(f"Received: {reply}")
+        _LOGGER.debug("%s: Received: %s", self.host, reply)
         return reply
 
 
@@ -345,7 +342,7 @@ class AsyncKefSpeaker:
     maximum_volume : float, optional
         The maximum allow volume, between 0 and 1. Use this to avoid
         accidentally setting very high volumes, by default 1.0.
-    ioloop : `asyncio.BaseEventLoop`, optional
+    loop : `asyncio.BaseEventLoop`, optional
         The eventloop to use.
     standby_time: int, optional
         Put the speaker in standby when inactive for ``standby_time``
@@ -369,7 +366,7 @@ class AsyncKefSpeaker:
         standby_time: Optional[int] = None,
         inverse_speaker_mode: bool = False,
         *,
-        ioloop: Optional[asyncio.events.AbstractEventLoop] = None,
+        loop: Optional[asyncio.events.AbstractEventLoop] = None,
     ):
         if standby_time not in STANDBY_OPTIONS:
             raise ValueError(
@@ -381,7 +378,7 @@ class AsyncKefSpeaker:
         self.maximum_volume = maximum_volume
         self.standby_time = standby_time
         self.inverse_speaker_mode = inverse_speaker_mode
-        self._comm = _AsyncCommunicator(host, port, ioloop=ioloop)
+        self._comm = _AsyncCommunicator(host, port, loop=loop)
         self.sync = SyncKefSpeaker(self)
 
     @retry(**_CMD_RETRY_KWARGS)
@@ -420,11 +417,15 @@ class AsyncKefSpeaker:
                 and ("R/L" if self.inverse_speaker_mode else "L/R")
                 and (state.standby_time == self.standby_time)
             ):
-                _LOGGER.debug(f"Source is {source}")
+                _LOGGER.debug("%s: Source is %s", self.host, source)
                 return
             _LOGGER.debug(
-                f"Try #{i}: Source is {current_source} but {source} is selected"
-            )
+                "%s: Try #%s: Source is %s but %s is selected",
+                self.host,
+                i,
+                current_source,
+                source,
+            ), self.host
             await asyncio.sleep(0.5)
 
         raise TimeoutError(
@@ -660,9 +661,11 @@ class AsyncKefSpeaker:
 
         for i in range(20):  # it can take 20s to boot
             if await self.is_on():
-                _LOGGER.debug("Speaker is on")
+                _LOGGER.debug("%s: Speaker is on", self.host)
                 return
-            _LOGGER.debug(f"Try #{i}: Turned on the speaker, but it is still off")
+            _LOGGER.debug(
+                "%s: Try #%s: Turned on the speaker, but it is still off", self.host, i
+            )
             await asyncio.sleep(1)
 
     async def turn_off(self) -> None:
@@ -673,9 +676,11 @@ class AsyncKefSpeaker:
 
         for i in range(20):  # it can take 20s to boot
             if not await self.is_on():
-                _LOGGER.debug("Speaker is off")
+                _LOGGER.debug("%s: Speaker is off", self.host)
                 return
-            _LOGGER.debug(f"Try #{i}: Turned off the speaker, but it is still on")
+            _LOGGER.debug(
+                "%s: Try #%s: Turned off the speaker, but it is still on", self.host, i
+            )
             await asyncio.sleep(1)
 
 
